@@ -28,10 +28,7 @@
 // ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //-------------------------------------------------------------------------
-extern crate static_math;
-
-use static_math::quaternion::Quaternion;
-use static_math::vector3::V3;
+use static_math::{V3, Quaternion};
 
 // In this example we rotate the x axis around the z axis 360 degrees
 // to obtain the x axis again, but the rotation is via a composition of
@@ -42,7 +39,8 @@ fn main() {
     let x = V3::x_axis();
     // quaternion represent the rotation around the z axis 90 degrees, the angle
     // is encoded in the vector norm: [0, 0, 90]
-    let q = Quaternion::rotation_norm_encoded(V3::z_axis() * 90.0f32.to_radians());
+    let v = V3::z_axis() * 90f32.to_radians();
+    let q = Quaternion::rotation_norm_encoded(&v);
     let r = q * q * q * q * x;
     println!("r: {:}", r);
     //-------------------------------------------------------------------------
