@@ -22,12 +22,8 @@
 //
 // You should have received a copy of the GNU General Public License
 //--------------------------------------------------------------------------
-#[macro_use]
-extern crate static_math;
-
-use static_math::matrix6x6::M66;
+use static_math::{M66, V6, m66_new};
 use static_math::traits::LinearAlgebra;
-use static_math::vector6::V6;
 
 fn main() {
 
@@ -46,6 +42,4 @@ fn main() {
         println!("verification: a * solution = b?: {}", a * solution);
         println!("b: {}", b);
     }
-
 }
-
