@@ -31,14 +31,14 @@
 // imports
 #![macro_use]
 use std::fmt;
-use std::ops::{Add, Mul, Sub, AddAssign, SubAssign};
+use std::ops::{Add, Mul, Sub, Div,AddAssign, SubAssign, Neg};
 use std::ops::{Deref, DerefMut, Index, IndexMut};
 
-use num::{Float, Num, Zero, One};
+use num::{Float, Num, Zero, One, Signed};
 use crate::vector2::*;
 use crate::traits::LinearAlgebra;
 use crate::slices_methods::*;
-use crate::utils::nearly_equal;
+use crate::utils::nearly_zero;
 
 /// A static Matrix of 2x2 shape
 #[derive(Copy, Clone, Debug, PartialEq)]
@@ -46,22 +46,27 @@ pub struct M22<T>([[T; 2]; 2]);
 
 impl<T> M22<T> {
 
+    #[inline(always)]
     pub const fn new(data_input: [[T; 2]; 2]) -> Self {
         Self(data_input)
     }
 
-    pub const fn create(a: T, b: T, c: T, d: T) -> Self {
+    #[inline(always)]
+    pub const fn new_from(a: T, b: T, c: T, d: T) -> Self {
         Self::new([[a, b], [c, d]])
     }
 
+    #[inline(always)]
     pub const fn rows(&self) -> usize {
         self.0.len()
     }
 
+    #[inline(always)]
     pub const fn cols(&self) -> usize {
         self.rows()
     }
 
+
 }
 
 impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M22<T> {
@@ -73,18 +78,22 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M22<T> {
         self.rows()
     }
 
+    #[inline(always)]
     fn det(&self) -> T {
         (self[(0, 0)] * self[(1, 1)]) - (self[(1, 0)] * self[(0, 1)])
     }
 
+    #[inline(always)]
     fn transpose(&self) -> M22<T> {
         M22::new([[self[(0, 0)], self[(1, 0)]], [self[(0, 1)], self[(1, 1)]]])
     }
 
+    #[inline(always)]
     fn trace(&self) -> T {
         self[(0, 0)] + self[(1, 1)]
     }
 
+    #[inline(always)]
     fn norm2(&self) -> T {
         let a = self[(0, 0)];
         let b = self[(0, 1)];
@@ -93,13 +102,14 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M22<T> {
         T::sqrt(a * a + b * b + c * c + d * d)
     }
 
+    #[inline(always)]
     fn inverse(&self) -> Option<Self> {
         let a = self[(0, 0)];
         let b = self[(0, 1)];
         let c = self[(1, 0)];
         let d = self[(1, 1)];
         let det = self.det();
-        if !nearly_equal(det, T::zero(), T::epsilon()) {
+        if !nearly_zero(det) {
             Some(M22::new([[d / det, -b / det], [-c / det, a / det]]))
         } else {
             None
@@ -109,7 +119,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M22<T> {
     /// Calculate de QR factorization of the M22 via gram-schmidt
     /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
-        if !nearly_equal(self.det(), T::zero(), T::epsilon()) {
+        if !nearly_zero(self.det()) {
             let cols = self.get_cols();
             let mut q: [V2<T>; 2] = *M22::zeros().get_cols();
             for i in 0..q.len() {
@@ -176,12 +186,14 @@ impl<T: Num + Copy> M22<T> {
         }
         result
     }
+
 }
 
 // M22 * V2
 impl<T: Num + Copy> Mul<V2<T>> for M22<T> {
     type Output = V2<T>;
 
+    #[inline(always)]
     fn mul(self, rhs: V2<T>) -> V2<T> {
         let a1 = self[(0, 0)];
         let b1 = self[(0, 1)];
@@ -198,6 +210,7 @@ impl<T: Num + Copy> Mul<V2<T>> for M22<T> {
 impl<T: Num + Copy> Add for M22<T> {
     type Output = Self;
 
+    #[inline(always)]
     fn add(self, rhs: Self) -> Self {
         let a1 = self[(0, 0)];
         let b1 = self[(0, 1)];
@@ -214,6 +227,7 @@ impl<T: Num + Copy> Add for M22<T> {
 
 // M22 += M22
 impl<T: Num + Copy> AddAssign for M22<T> {
+    #[inline(always)]
     fn add_assign(&mut self, other: Self) {
         *self = *self + other
     }
@@ -223,6 +237,7 @@ impl<T: Num + Copy> AddAssign for M22<T> {
 impl<T: Num + Copy> Sub for M22<T> {
     type Output = Self;
 
+    #[inline(always)]
     fn sub(self, rhs: Self) -> Self {
         let a1 = self[(0, 0)];
         let b1 = self[(0, 1)];
@@ -322,6 +337,7 @@ impl Mul<M22<f32>> for f32 {
 impl<T: Num + Copy> Mul<T> for M22<T> {
     type Output = M22<T>;
 
+    #[inline(always)]
     fn mul(self, rhs: T) -> M22<T> {
         let a_00 = self[(0, 0)] * rhs;
         let a_01 = self[(0, 1)] * rhs;
@@ -332,10 +348,25 @@ impl<T: Num + Copy> Mul<T> for M22<T> {
     }
 }
 
+// M22 / constant
+impl<T: Num + Copy> Div<T> for M22<T> {
+    type Output = Self;
+
+    fn div(self, rhs: T) -> Self::Output {
+        let a_00 = self[(0, 0)] / rhs;
+        let a_01 = self[(0, 1)] / rhs;
+        let a_10 = self[(1, 0)] / rhs;
+        let a_11 = self[(1, 1)] / rhs;
+
+        M22::new([[a_00, a_01], [a_10, a_11]])
+    }
+}
+
 // M22 * M22
 impl<T: Num + Copy> Mul for M22<T> {
     type Output = Self;
 
+    #[inline(always)]
     fn mul(self, rhs: Self) -> Self {
         let a1 = self[(0, 0)];
         let b1 = self[(0, 1)];
@@ -356,7 +387,21 @@ impl<T: Num + Copy> Mul for M22<T> {
     }
 }
 
+// -M22
+impl<T: Num + Copy + Signed> Neg for M22<T> {
+    type Output = Self;
+
+    fn neg(self) -> Self {
+        let a = -self[(0, 0)];
+        let b = -self[(0, 1)];
+        let c = -self[(1, 0)];
+        let d = -self[(1, 1)];
+        M22::new([[a, b], [c, d]])
+    }
+}
+
 impl<T: Num + Copy> Zero for M22<T> {
+    #[inline(always)]
     fn zero() -> M22<T> {
         M22::new([[T::zero(); 2]; 2])
     }
@@ -426,7 +471,7 @@ macro_rules! m22_new {
 //-------------------------------------------------------------------------
 impl<T: Num + fmt::Display> fmt::Display for M22<T> {
     fn fmt(&self, dest: &mut fmt::Formatter) -> fmt::Result {
-        println!("");
+        println!();
         write!(dest, "|{0:^3.2} {1:^3.2}|\n", self[(0, 0)], self[(0, 1)])?;
         write!(dest, "|{0:^3.2} {1:^3.2}|\n", self[(1, 0)], self[(1, 1)])
     }
