@@ -30,12 +30,12 @@
 //-------------------------------------------------------------------------
 #![macro_use]
 use std::fmt;
-use std::ops::{Add, Mul, Sub, SubAssign, AddAssign};
+use std::ops::{Add, Mul, Div, Sub, SubAssign, AddAssign, Neg};
 use std::ops::{Deref, DerefMut, Index, IndexMut};
 
 use crate::traits::LinearAlgebra;
-use crate::utils::nearly_equal;
-use num::{Float, One, Zero, Num};
+use crate::utils::nearly_zero;
+use num::{Float, One, Zero, Num, Signed};
 
 use crate::slices_methods::*;
 use crate::vector3::*;
@@ -70,6 +70,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M33<T> {
         self.rows()
     }
 
+    #[inline(always)]
     fn transpose(&self) -> M33<T> {
         M33::new([
             [self[(0, 0)], self[(1, 0)], self[(2, 0)]],
@@ -97,25 +98,18 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M33<T> {
     }
 
     /// Calculate the determiant of the matrix
+    #[inline(always)]
     fn det(&self) -> T {
-        let a_00 = self[(0, 0)];
-        let a_01 = self[(0, 1)];
-        let a_02 = self[(0, 2)];
-        let a_10 = self[(1, 0)];
-        let a_11 = self[(1, 1)];
-        let a_12 = self[(1, 2)];
-        let a_20 = self[(2, 0)];
-        let a_21 = self[(2, 1)];
-        let a_22 = self[(2, 2)];
-        a_00 * (a_11 * a_22 - a_21 * a_12)
-            - a_01 * (a_10 * a_22 - a_12 * a_20)
-            + a_02 * (a_10 * a_21 - a_11 * a_20)
+          self[(0, 0)] * (self[(1, 1)] * self[(2, 2)] - self[(2, 1)] * self[(1, 2)])
+        - self[(0, 1)] * (self[(1, 0)] * self[(2, 2)] - self[(1, 2)] * self[(2, 0)])
+        + self[(0, 2)] * (self[(1, 0)] * self[(2, 1)] - self[(1, 1)] * self[(2, 0)])
     }
 
     /// Calculate the inverse
+    #[inline(always)]
     fn inverse(&self) -> Option<Self> {
         let det = self.det();
-        if !nearly_equal(det, T::zero(), T::epsilon()) {
+        if !nearly_zero(det) {
             let invdet = T::one() / det;
             let mut res = M33::zero();
             res[(0, 0)] = (self[(1, 1)] * self[(2, 2)] - self[(2, 1)] * self[(1, 2)]) * invdet;
@@ -136,7 +130,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M33<T> {
     /// Calculate de QR factorization of the M33 via gram-schmidt
     /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
-        if !nearly_equal(self.det(), T::zero(), T::epsilon()) {
+        if !nearly_zero(self.det()) {
             let cols = self.get_cols();
             let mut q: [V3<T>; 3] = *M33::zeros().get_cols();
             for i in 0..q.len() {
@@ -282,6 +276,7 @@ impl<T: Num + Copy> M33<T> {
 impl<T: Num + Copy> Add for M33<T> {
     type Output = Self;
 
+    #[inline(always)]
     fn add(self, rhs: Self) -> Self {
         M33::new([
             [
@@ -346,6 +341,7 @@ impl<T: Num + Copy> SubAssign for M33<T> {
 impl<T: Num + Copy> Mul<V3<T>> for M33<T> {
     type Output = V3<T>;
 
+    #[inline(always)]
     fn mul(self, rhs: V3<T>) -> V3<T> {
 
         let a_00 = self[(0, 0)];
@@ -386,10 +382,69 @@ impl<T: Num + Copy> Mul<T> for M33<T> {
     }
 }
 
+// M33 / constant
+impl<T: Num + Copy> Div<T> for M33<T> {
+    type Output = Self;
+
+    fn div(self, rhs: T) -> Self::Output {
+        let a_00 = self[(0, 0)] / rhs;
+        let a_01 = self[(0, 1)] / rhs;
+        let a_02 = self[(0, 2)] / rhs;
+        let a_10 = self[(1, 0)] / rhs;
+        let a_11 = self[(1, 1)] / rhs;
+        let a_12 = self[(1, 2)] / rhs;
+        let a_20 = self[(2, 0)] / rhs;
+        let a_21 = self[(2, 1)] / rhs;
+        let a_22 = self[(2, 2)] / rhs;
+
+        M33::new([[a_00, a_01, a_02], [a_10, a_11, a_12], [a_20, a_21, a_22]])
+    }
+}
+
+// f32 * M33<f32>
+impl Mul<M33<f32>> for f32 {
+    type Output = M33<f32>;
+
+    fn mul(self, rhs: M33<f32>) -> Self::Output {
+        let a_00 = self * rhs[(0, 0)];
+        let a_01 = self * rhs[(0, 1)];
+        let a_02 = self * rhs[(0, 2)];
+        let a_10 = self * rhs[(1, 0)];
+        let a_11 = self * rhs[(1, 1)];
+        let a_12 = self * rhs[(1, 2)];
+        let a_20 = self * rhs[(2, 0)];
+        let a_21 = self * rhs[(2, 1)];
+        let a_22 = self * rhs[(2, 2)];
+
+        M33::new([[a_00, a_01, a_02], [a_10, a_11, a_12], [a_20, a_21, a_22]])
+    }
+}
+
+// f64 * M33<f64>
+impl Mul<M33<f64>> for f64 {
+    type Output = M33<f64>;
+
+    fn mul(self, rhs: M33<f64>) -> Self::Output {
+        let a_00 = self * rhs[(0, 0)];
+        let a_01 = self * rhs[(0, 1)];
+        let a_02 = self * rhs[(0, 2)];
+        let a_10 = self * rhs[(1, 0)];
+        let a_11 = self * rhs[(1, 1)];
+        let a_12 = self * rhs[(1, 2)];
+        let a_20 = self * rhs[(2, 0)];
+        let a_21 = self * rhs[(2, 1)];
+        let a_22 = self * rhs[(2, 2)];
+
+        M33::new([[a_00, a_01, a_02], [a_10, a_11, a_12], [a_20, a_21, a_22]])
+    }
+}
+
+
 // M3 * M3
 impl<T: Num + Copy> Mul for M33<T> {
     type Output = Self;
 
+    #[inline]
     fn mul(self, rhs: Self) -> Self {
         let m00 =
             self[(0, 0)] * rhs[(0, 0)] + self[(0, 1)] * rhs[(1, 0)] + self[(0, 2)] * rhs[(2, 0)];
@@ -416,6 +471,25 @@ impl<T: Num + Copy> Mul for M33<T> {
     }
 }
 
+// -M33
+impl<T: Num + Copy + Signed> Neg for M33<T> {
+    type Output = Self;
+
+    fn neg(self) -> Self {
+        let a_00 = -self[(0, 0)];
+        let a_01 = -self[(0, 1)];
+        let a_02 = -self[(0, 2)];
+        let a_10 = -self[(1, 0)];
+        let a_11 = -self[(1, 1)];
+        let a_12 = -self[(1, 2)];
+        let a_20 = -self[(2, 0)];
+        let a_21 = -self[(2, 1)];
+        let a_22 = -self[(2, 2)];
+
+        M33::new([[a_00, a_01, a_02], [a_10, a_11, a_12], [a_20, a_21, a_22]])
+    }
+}
+
 impl<T: Num + Copy> Zero for M33<T> {
     fn zero() -> M33<T> {
         M33::new([[T::zero(); 3]; 3])
@@ -476,7 +550,7 @@ impl<T> IndexMut<(usize, usize)> for M33<T> {
 //-------------------------------------------------------------------------
 impl<T: Num + fmt::Display> fmt::Display for M33<T> {
     fn fmt(&self, dest: &mut fmt::Formatter) -> fmt::Result {
-        println!("");
+        println!();
         write!(
             dest,
             "|{0:<7.2} {1:^7.2} {2:>7.2}|\n",
