@@ -29,15 +29,15 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //-------------------------------------------------------------------------
 use std::fmt;
-use std::ops::{Add, Mul, Sub, AddAssign, SubAssign};
+use std::ops::{Add, Mul, Div, Sub, AddAssign, SubAssign, Neg};
 use std::ops::{Deref, DerefMut, Index, IndexMut};
 
-use num::{Float, One, Zero, Num};
+use num::{Float, One, Zero, Num, Signed};
 use crate::matrix3x3::*;
 use crate::slices_methods::*;
 use crate::traits::LinearAlgebra;
 use crate::vector4::V4;
-use crate::utils::nearly_equal;
+use crate::utils::nearly_zero;
 
 //-------------------------------------------------------------------------
 //                        code
@@ -69,6 +69,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M44<T> {
         self.rows()
     }
 
+    #[inline(always)]
     fn transpose(&self) -> M44<T> {
         M44::new([
             [self[(0, 0)], self[(1, 0)], self[(2, 0)], self[(3, 0)]],
@@ -78,6 +79,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M44<T> {
         ])
     }
 
+    #[inline(always)]
     fn trace(&self) -> T {
         return self[(0, 0)] + self[(1, 1)] + self[(2, 2)] + self[(3, 3)];
     }
@@ -119,6 +121,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M44<T> {
         )
     }
 
+    #[inline(always)]
     fn det(&self) -> T {
         let a1 = self[(0, 0)];
         let a2 = self[(0, 1)];
@@ -166,32 +169,32 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M44<T> {
     /// Calculate the inverse
     fn inverse(&self) -> Option<Self> {
         let det = self.det();
-        if !nearly_equal(det, T::zero(), T::epsilon()) {
-            let a1 = self.get_submatrix((0, 0)).det();
-            let a2 = -self.get_submatrix((1, 0)).det();
-            let a3 = self.get_submatrix((2, 0)).det();
-            let a4 = -self.get_submatrix((3, 0)).det();
+        if !nearly_zero(det) {
+            let a1 = self.get_submatrix((0, 0)).det() / det;
+            let a2 = -self.get_submatrix((1, 0)).det() / det;
+            let a3 = self.get_submatrix((2, 0)).det() / det;
+            let a4 = -self.get_submatrix((3, 0)).det() / det;
 
-            let a5 = -self.get_submatrix((0, 1)).det();
-            let a6 = self.get_submatrix((1, 1)).det();
-            let a7 = -self.get_submatrix((2, 1)).det();
-            let a8 = self.get_submatrix((3, 1)).det();
+            let a5 = -self.get_submatrix((0, 1)).det() / det;
+            let a6 = self.get_submatrix((1, 1)).det() / det;
+            let a7 = -self.get_submatrix((2, 1)).det() / det;
+            let a8 = self.get_submatrix((3, 1)).det() / det;
 
-            let a9 = self.get_submatrix((0, 2)).det();
-            let a10 = -self.get_submatrix((1, 2)).det();
-            let a11 = self.get_submatrix((2, 2)).det();
-            let a12 = -self.get_submatrix((3, 2)).det();
+            let a9 = self.get_submatrix((0, 2)).det() / det;
+            let a10 = -self.get_submatrix((1, 2)).det() / det;
+            let a11 = self.get_submatrix((2, 2)).det() / det;
+            let a12 = -self.get_submatrix((3, 2)).det() / det;
 
-            let a13 = -self.get_submatrix((0, 3)).det();
-            let a14 = self.get_submatrix((1, 3)).det();
-            let a15 = -self.get_submatrix((2, 3)).det();
-            let a16 = self.get_submatrix((3, 3)).det();
+            let a13 = -self.get_submatrix((0, 3)).det() / det;
+            let a14 = self.get_submatrix((1, 3)).det() / det;
+            let a15 = -self.get_submatrix((2, 3)).det() / det;
+            let a16 = self.get_submatrix((3, 3)).det() / det;
 
             Some(M44::new([
-                [a1 / det, a2 / det, a3 / det, a4 / det],
-                [a5 / det, a6 / det, a7 / det, a8 / det],
-                [a9 / det, a10 / det, a11 / det, a12 / det],
-                [a13 / det, a14 / det, a15 / det, a16 / det],
+                [a1, a2 , a3 , a4],
+                [a5, a6 , a7 , a8],
+                [a9, a10, a11, a12],
+                [a13, a14, a15, a16],
             ]))
         } else {
             None
@@ -201,7 +204,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M44<T> {
     /// Calculate de QR factorization of the M44 via gram-schmidt
     /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
-        if !nearly_equal(self.det(), T::zero(), T::epsilon()) {
+        if !nearly_zero(self.det()) {
             let cols = self.get_cols();
             let mut q: [V4<T>; 4] = *M44::zeros().get_cols();
             for i in 0..q.len() {
@@ -271,6 +274,8 @@ impl<T: Num + Copy> M44<T> {
         result
     }
 
+    // NOTE(elsuizo:2021-04-30): with this inline the performance its worse
+    // #[inline(always)]
     pub fn get_submatrix(&self, selected: (usize, usize)) -> M33<T> {
         let mut values: [T; 9] = [T::zero(); 9];
         let mut result: M33<T> = M33::zeros();
@@ -504,6 +509,37 @@ impl<T: Num + Copy> Mul<T> for M44<T> {
     }
 }
 
+// M44 / constant
+impl<T: Num + Copy> Div<T> for M44<T> {
+    type Output = Self;
+
+    fn div(self, rhs: T) -> Self::Output {
+        let a1 = self[(0, 0)] / rhs;
+        let a2 = self[(0, 1)] / rhs;
+        let a3 = self[(0, 2)] / rhs;
+        let a4 = self[(0, 3)] / rhs;
+        let a5 = self[(1, 0)] / rhs;
+        let a6 = self[(1, 1)] / rhs;
+        let a7 = self[(1, 2)] / rhs;
+        let a8 = self[(1, 3)] / rhs;
+        let a9 = self[(2, 0)] / rhs;
+        let a10 = self[(2, 1)] / rhs;
+        let a11 = self[(2, 2)] / rhs;
+        let a12 = self[(2, 3)] / rhs;
+        let a13 = self[(3, 0)] / rhs;
+        let a14 = self[(3, 1)] / rhs;
+        let a15 = self[(3, 2)] / rhs;
+        let a16 = self[(3, 3)] / rhs;
+
+        M44::new([
+            [a1, a2, a3, a4],
+            [a5, a6, a7, a8],
+            [a9, a10, a11, a12],
+            [a13, a14, a15, a16],
+        ])
+    }
+}
+
 // M44 * M44
 impl<T: Num + Copy> Mul for M44<T> {
     type Output = Self;
@@ -570,6 +606,36 @@ impl<T: Num + Copy> Mul for M44<T> {
     }
 }
 
+// -M44
+impl<T: Num + Copy + Signed> Neg for M44<T> {
+    type Output = Self;
+
+    fn neg(self) -> Self {
+        let a1  = -self[(0, 0)];
+        let a2  = -self[(0, 1)];
+        let a3  = -self[(0, 2)];
+        let a4  = -self[(0, 3)];
+        let a5  = -self[(1, 0)];
+        let a6  = -self[(1, 1)];
+        let a7  = -self[(1, 2)];
+        let a8  = -self[(1, 3)];
+        let a9  = -self[(2, 0)];
+        let a10 = -self[(2, 1)];
+        let a11 = -self[(2, 2)];
+        let a12 = -self[(2, 3)];
+        let a13 = -self[(3, 0)];
+        let a14 = -self[(3, 1)];
+        let a15 = -self[(3, 2)];
+        let a16 = -self[(3, 3)];
+
+        M44::new([
+            [a1, a2, a3, a4],
+            [a5, a6, a7, a8],
+            [a9, a10, a11, a12],
+            [a13, a14, a15, a16],
+        ])
+    }
+}
 // M44 * V4
 impl<T: Num + Copy> Mul<V4<T>> for M44<T> {
     type Output = V4<T>;
@@ -674,7 +740,7 @@ impl<T> IndexMut<(usize, usize)> for M44<T> {
 //-------------------------------------------------------------------------
 impl<T: Num + fmt::Display> fmt::Display for M44<T> {
     fn fmt(&self, dest: &mut fmt::Formatter) -> fmt::Result {
-        println!("");
+        println!();
         write!(
             dest,
             "|{0:<7.2} {1:^7.2} {2:^7.2} {3:>7.2}|\n",
