@@ -29,16 +29,17 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //-------------------------------------------------------------------------
 use std::fmt;
-use std::ops::{Add, Mul, Sub, AddAssign, SubAssign};
+use std::ops::{Add, Mul, Div, Sub, AddAssign, SubAssign, Neg};
 use std::ops::{Deref, DerefMut, Index, IndexMut};
 
-use num::{Float, Num, One, Zero};
+use num::{Float, Num, One, Zero, Signed};
 
 use crate::slices_methods::*;
 use crate::matrix5x5::M55;
 use crate::vector6::V6;
+use crate::matrix3x3::M33;
 use crate::traits::LinearAlgebra;
-use crate::utils::nearly_equal;
+use crate::utils::nearly_zero;
 //-------------------------------------------------------------------------
 //                        code
 //-------------------------------------------------------------------------
@@ -119,14 +120,9 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M66<T> {
         ])
     }
 
-    #[inline]
+    #[inline(always)]
     fn trace(&self) -> T {
-        return self[(0, 0)]
-            + self[(1, 1)]
-            + self[(2, 2)]
-            + self[(3, 3)]
-            + self[(4, 4)]
-            + self[(5, 5)];
+           self[(0, 0)] + self[(1, 1)] + self[(2, 2)] + self[(3, 3)] + self[(4, 4)] + self[(5, 5)]
     }
 
     fn norm2(&self) -> T {
@@ -139,7 +135,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M66<T> {
         )
     }
 
-    #[inline]
+    #[inline(always)]
     fn det(&self) -> T {
         let a_00 = self[(0, 0)];
         let a_01 = self[(0, 1)];
@@ -907,10 +903,11 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M66<T> {
     /// where: Adj = Cofactor.Transpose()
     ///
     /// Cofactor = (-1)^(i+j) * M(i, j).det()
+    #[inline(always)]
     fn inverse(&self) -> Option<Self> {
         let one = T::one();
         let det = self.det();
-        if !nearly_equal(det, T::zero(), T::epsilon()) {
+        if !nearly_zero(det) {
             let mut cofactors: M66<T> = M66::zeros();
             for i in 0..6 {
                 for j in 0..6 {
@@ -919,7 +916,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M66<T> {
                     cofactors[(j, i)] = sign * self.get_submatrix((i, j)).det();
                 }
             }
-            Some(cofactors * (one / det))
+            Some(cofactors / det)
         } else {
             None
         }
@@ -928,7 +925,7 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M66<T> {
     /// Calculate de QR factorization of the M66 via gram-schmidt
     /// orthogonalization process
     fn qr(&self) -> Option<(Self, Self)> {
-        if !nearly_equal(self.det(), T::zero(), T::epsilon()) {
+        if !nearly_zero(self.det()) {
             let cols = self.get_cols();
             let mut q: [V6<T>; 6] = *M66::zeros().get_cols();
             for i in 0..q.len() {
@@ -1300,6 +1297,112 @@ impl<T: Num + Copy> Mul<T> for M66<T> {
     }
 }
 
+// M66 / constant
+impl<T: Num + Copy> Div<T> for M66<T> {
+    type Output = Self;
+
+    fn div(self, rhs: T) -> Self::Output {
+        let a_00 = self[(0, 0)] / rhs;
+        let a_01 = self[(0, 1)] / rhs;
+        let a_02 = self[(0, 2)] / rhs;
+        let a_03 = self[(0, 3)] / rhs;
+        let a_04 = self[(0, 4)] / rhs;
+        let a_05 = self[(0, 5)] / rhs;
+        let a_10 = self[(1, 0)] / rhs;
+        let a_11 = self[(1, 1)] / rhs;
+        let a_12 = self[(1, 2)] / rhs;
+        let a_13 = self[(1, 3)] / rhs;
+        let a_14 = self[(1, 4)] / rhs;
+        let a_15 = self[(1, 5)] / rhs;
+        let a_20 = self[(2, 0)] / rhs;
+        let a_21 = self[(2, 1)] / rhs;
+        let a_22 = self[(2, 2)] / rhs;
+        let a_23 = self[(2, 3)] / rhs;
+        let a_24 = self[(2, 4)] / rhs;
+        let a_25 = self[(2, 5)] / rhs;
+        let a_30 = self[(3, 0)] / rhs;
+        let a_31 = self[(3, 1)] / rhs;
+        let a_32 = self[(3, 2)] / rhs;
+        let a_33 = self[(3, 3)] / rhs;
+        let a_34 = self[(3, 4)] / rhs;
+        let a_35 = self[(3, 5)] / rhs;
+        let a_40 = self[(4, 0)] / rhs;
+        let a_41 = self[(4, 1)] / rhs;
+        let a_42 = self[(4, 2)] / rhs;
+        let a_43 = self[(4, 3)] / rhs;
+        let a_44 = self[(4, 4)] / rhs;
+        let a_45 = self[(4, 5)] / rhs;
+        let a_50 = self[(5, 0)] / rhs;
+        let a_51 = self[(5, 1)] / rhs;
+        let a_52 = self[(5, 2)] / rhs;
+        let a_53 = self[(5, 3)] / rhs;
+        let a_54 = self[(5, 4)] / rhs;
+        let a_55 = self[(5, 5)] / rhs;
+
+        M66::new([
+            [a_00, a_01, a_02, a_03, a_04, a_05],
+            [a_10, a_11, a_12, a_13, a_14, a_15],
+            [a_20, a_21, a_22, a_23, a_24, a_25],
+            [a_30, a_31, a_32, a_33, a_34, a_35],
+            [a_40, a_41, a_42, a_43, a_44, a_45],
+            [a_50, a_51, a_52, a_53, a_54, a_55],
+        ])
+    }
+}
+
+// -M66
+impl<T: Num + Copy + Signed> Neg for M66<T> {
+    type Output = Self;
+
+    fn neg(self) -> Self {
+        let a_00 = -self[(0, 0)];
+        let a_01 = -self[(0, 1)];
+        let a_02 = -self[(0, 2)];
+        let a_03 = -self[(0, 3)];
+        let a_04 = -self[(0, 4)];
+        let a_05 = -self[(0, 5)];
+        let a_10 = -self[(1, 0)];
+        let a_11 = -self[(1, 1)];
+        let a_12 = -self[(1, 2)];
+        let a_13 = -self[(1, 3)];
+        let a_14 = -self[(1, 4)];
+        let a_15 = -self[(1, 5)];
+        let a_20 = -self[(2, 0)];
+        let a_21 = -self[(2, 1)];
+        let a_22 = -self[(2, 2)];
+        let a_23 = -self[(2, 3)];
+        let a_24 = -self[(2, 4)];
+        let a_25 = -self[(2, 5)];
+        let a_30 = -self[(3, 0)];
+        let a_31 = -self[(3, 1)];
+        let a_32 = -self[(3, 2)];
+        let a_33 = -self[(3, 3)];
+        let a_34 = -self[(3, 4)];
+        let a_35 = -self[(3, 5)];
+        let a_40 = -self[(4, 0)];
+        let a_41 = -self[(4, 1)];
+        let a_42 = -self[(4, 2)];
+        let a_43 = -self[(4, 3)];
+        let a_44 = -self[(4, 4)];
+        let a_45 = -self[(4, 5)];
+        let a_50 = -self[(5, 0)];
+        let a_51 = -self[(5, 1)];
+        let a_52 = -self[(5, 2)];
+        let a_53 = -self[(5, 3)];
+        let a_54 = -self[(5, 4)];
+        let a_55 = -self[(5, 5)];
+
+    M66::new([
+        [a_00, a_01, a_02, a_03, a_04, a_05],
+        [a_10, a_11, a_12, a_13, a_14, a_15],
+        [a_20, a_21, a_22, a_23, a_24, a_25],
+        [a_30, a_31, a_32, a_33, a_34, a_35],
+        [a_40, a_41, a_42, a_43, a_44, a_45],
+        [a_50, a_51, a_52, a_53, a_54, a_55],
+    ])
+    }
+}
+
 // M66 * V6
 impl<T: Num + Copy> Mul<V6<T>> for M66<T> {
     type Output = V6<T>;
@@ -1644,6 +1747,53 @@ impl<T: Num + Copy> M66<T> {
         }
         result
     }
+
+    /// Copy elements from a M33<T> matrix in one of the 4 cuadrants of the actual
+    /// matrix, where the quadrants are enumerated as:
+    /// +---+---+
+    /// | 1 | 2 |
+    /// +---+---+
+    /// | 3 | 4 |
+    /// +---+---+
+    ///
+    /// Function arguments:
+    /// `r`: M33<T> submatrix elements to copy
+    ///
+    /// `quadrant`: number to choose the cuadrant where do the copy of the elements
+    ///
+    pub fn copy_elements_from(&mut self, r: &M33<T>, quadrant: usize) {
+        match quadrant {
+            1 => {
+                for i in 0..3 {
+                    for j in 0..3 {
+                        self[(i, j)] = r[(i, j)];
+                    }
+                }
+            },
+            2 => {
+                for i in 0..3 {
+                    for j in 3..6 {
+                        self[(i, j)] = r[(i, j-3)];
+                    }
+                }
+            },
+            3 => {
+                for i in 3..6 {
+                    for j in 0..3 {
+                        self[(i, j)] = r[(i-3, j)];
+                    }
+                }
+            },
+            4 => {
+                for i in 3..6 {
+                    for j in 3..6 {
+                        self[(i, j)] = r[(i-3, j-3)];
+                    }
+                }
+            },
+            _ => panic!("no more than 4 quadrants !!!")
+        }
+    }
 }
 
 impl<T: Num + Copy> Zero for M66<T> {
@@ -1734,7 +1884,7 @@ macro_rules! m66_new {
 //-------------------------------------------------------------------------
 impl<T: Num + fmt::Display> fmt::Display for M66<T> {
     fn fmt(&self, dest: &mut fmt::Formatter) -> fmt::Result {
-        println!("");
+        println!();
         write!(
             dest,
             "|{0:<7.2} {1:^7.2} {2:^7.2} {3:^7.2} {4:^7.2} {5:>7.2}|\n",
