@@ -33,30 +33,34 @@ use std::ops::{Mul, Add, Sub, Neg, Div};
 use num::{Num, Float, Signed, Zero, One};
 use num::traits::FloatConst;
 use crate::vector3::*;
+use crate::vector4::V4;
 use crate::matrix3x3::M33;
 use crate::transformations::rotation_to_euler;
-use crate::utils::nearly_equal;
+use crate::utils::nearly_zero;
+use crate::traits::LinearAlgebra;
 
 /// Quaternion type
 #[derive(Copy, Debug, Clone, PartialEq)]
 pub struct Quaternion<T> {
     /// Scalar part
-    pub q0: T,
+    q0: T,
     /// Imaginary part
-    pub q: V3<T>,
+    q: V3<T>,
     /// flag to signaling if the Quaternion is normalized
     normalized: bool
 }
 
 impl<T> Quaternion<T> {
 
-    /// construct a new Quaternion from a number(real part) and a vector(imag part)
+    /// Construct a new Quaternion from a number(real part) and a vector(imag part)
+    #[inline(always)]
     pub const fn new(q0: T, q: V3<T>) -> Self {
         Self{q0, q, normalized: false}
     }
 
-    /// construct a new Quaternion zrom four numbers
-    pub fn new_from(q0: T, q1: T, q2: T, q3: T) -> Self {
+    /// Construct a new Quaternion from four numbers
+    #[inline(always)]
+    pub const fn new_from(q0: T, q1: T, q2: T, q3: T) -> Self {
         Self{q0, q: V3::new([q1, q2, q3]), normalized: false}
     }
 }
@@ -93,8 +97,8 @@ impl<T: Num + Copy> Quaternion<T> {
     }
 
     /// construct a pure "imaginary" Quaternion
-    pub fn new_imag(q: V3<T>) -> Self {
-        Self{q0: T::zero(), q, normalized: false}
+    pub fn new_imag(q: &V3<T>) -> Self {
+        Self{q0: T::zero(), q: *q, normalized: false}
     }
 
     /// calculate the abs2 of the Quaternion
@@ -102,8 +106,25 @@ impl<T: Num + Copy> Quaternion<T> {
         self.q0 * self.q0 + self.q[0] * self.q[0] + self.q[1] * self.q[1] + self.q[2] * self.q[2]
     }
 
-    
+    /// Construct a new Quternion from a V4
+    pub fn new_from_vec(v: &V4<T>) -> Self {
+        Self{q0: v[0], q: V3::new_from(v[1], v[2], v[3]), normalized: false}
+    }
 
+    /// convert the Quaternion to a rotation matrix
+    pub fn to_rotation(&self) -> M33<T> {
+        let (q0, q) = (self.real(), self.imag());
+        let q0_s = q0 * q0;
+        let (q1, q2, q3) = (q[0], q[1], q[2]);
+        let q1_s = q1 * q1;
+        let q2_s = q2 * q2;
+        let q3_s = q3 * q3;
+        let two = T::one() + T::one();
+
+        m33_new!(q0_s + q1_s - q2_s - q3_s, two*q1*q2 - two*q0*q3, two*q1*q3 + two*q0*q2;
+                 two*q1*q2 + two*q0*q3, q0_s - q1_s + q2_s - q3_s, two*q2*q3 - two*q0*q1;
+                 two*q1*q3 - two*q0*q2, two*q2*q3 + two*q0*q1, q0_s - q1_s - q2_s + q3_s)
+    }
 }
 
 // create the zero Quaternion
@@ -122,7 +143,7 @@ impl<T: Num + Copy> One for Quaternion<T> {
     /// Create an identity Quaternion
     fn one() -> Self {
         let one = T::one();
-        Self::new(one, V3::zeros())
+        Self{q0: one, q: V3::zeros(), normalized: true}
     }
 }
 
@@ -158,7 +179,7 @@ impl<T: Float + Signed> Div for Quaternion<T> {
     type Output = Self;
 
     fn div(self, rhs: Self) -> Self::Output {
-        self * rhs.inv().expect("the input has to be a non zero vector")
+        self * rhs.inverse().expect("the input has to be a non zero vector")
     }
 }
 
@@ -166,6 +187,7 @@ impl<T: Float + Signed> Div for Quaternion<T> {
 impl<T: Num + Copy> Mul for Quaternion<T> {
     type Output = Self;
 
+    #[inline(always)]
     fn mul(self, rhs: Self) -> Self::Output {
         let q0 = self.q0 * rhs.q0  - self.q * rhs.q;
         let q = rhs.q * self.q0 + self.q * rhs.q0 + self.q.cross(rhs.q);
@@ -178,6 +200,7 @@ impl<T: Num + Copy> Mul for Quaternion<T> {
 // from: Fabian “ryg” Giesen
 impl<T: Num + Copy + Signed> Mul<V3<T>> for Quaternion<T> {
     type Output = V3<T>;
+    #[inline(always)]
     fn mul(self, rhs: V3<T>) -> Self::Output {
         let one = T::one();
         let two = one + one;
@@ -186,6 +209,7 @@ impl<T: Num + Copy + Signed> Mul<V3<T>> for Quaternion<T> {
     }
 }
 
+// q * const
 impl<T: Num + Copy> Mul<T> for Quaternion<T> {
     type Output = Quaternion<T>;
     fn mul(self, rhs: T) -> Self::Output {
@@ -193,6 +217,7 @@ impl<T: Num + Copy> Mul<T> for Quaternion<T> {
     }
 }
 
+// -q
 impl<T: Num + Copy + Signed> Neg for Quaternion<T> {
     type Output = Self;
     #[inline]
@@ -207,32 +232,52 @@ impl<T: Num + Copy + Signed> Quaternion<T> {
     }
 }
 
-// NOTE(elsuizo:2021-04-23): we only need the Float for the sqrt function
-impl<T: Float> Quaternion<T> {
-    pub fn norm2(&self) -> T {
-        self.dot(*self).sqrt()
-    }
-}
+impl<T: Float + std::iter::Sum> Quaternion<T> {
 
-impl<T: Float + Signed> Quaternion<T> {
-    pub fn inv(&self) -> Option<Self> {
-        if !self.normalized {
-            let norm_sqr = self.norm2();
-            nearly_equal(norm_sqr, T::zero(), T::epsilon()).then(|| self.conj() / norm_sqr)
+    pub fn from_rotation(rot: &M33<T>) -> Self {
+        let m = rot.transpose();
+        let zero = T::zero();
+        let one  = T::one();
+        let half = one / (one + one);
+        if m[(2, 2)] < zero {
+            if m[(0, 0)] > m[(1, 1)] {
+                let t = one + m[(0, 0)] - m[(1, 1)] - m[(2, 2)];
+                let q = Self::new_from(m[(1, 2)]-m[(2, 1)],  t,  m[(0, 1)]+m[(1, 0)],  m[(2, 0)]+m[(0, 2)]);
+                return q * half / t.sqrt();
+            } else {
+                let t = one - m[(0, 0)] + m[(1, 1)] - m[(2, 2)];
+                let q = Self::new_from(m[(2, 0)]-m[(0, 2)],  m[(0, 1)]+m[(1, 0)],  t,  m[(1, 2)]+m[(2, 1)]);
+                return q * half / t.sqrt();
+            }
         } else {
-            Some(self.conj())
+            if m[(0, 0)] < -m[(1, 1)] {
+                let t = one - m[(0, 0)] - m[(1, 1)] + m[(2, 2)];
+                let q = Self::new_from(m[(0, 1)]-m[(1, 0)],  m[(2, 0)]+m[(0, 2)],  m[(1, 2)]+m[(2, 1)],  t);
+                return q * half / t.sqrt();
+            } else {
+                let t = one + m[(0, 0)] + m[(1, 1)] + m[(2, 2)];
+                let q = Self::new_from(t,  m[(1, 2)]-m[(2, 1)],  m[(2, 0)]-m[(0, 2)],  m[(0, 1)]-m[(1, 0)]);
+                return q * half / t.sqrt();
+            }
         }
     }
 }
 
-impl<T: Float + FloatConst + Signed> Quaternion<T> {
-    /// normalize the Quaternion only if necesary
+// NOTE(elsuizo:2021-04-23): we only need the Float for the sqrt function
+impl<T: Float> Quaternion<T> {
+
+    /// the euclidean norm of the Quaternion
+    pub fn norm2(&self) -> T {
+        self.dot(*self).sqrt()
+    }
+
+    /// normalize the Quaternion only if necessary
     pub fn normalize(&self) -> Option<Self> {
         if self.normalized {
             Some(*self)
         } else {
             let norm_sqr = self.norm2();
-            if norm_sqr > T::epsilon() {
+            if !nearly_zero(norm_sqr) {
                 let mut result = *self / norm_sqr;
                 result.normalized = true;
                 Some(result)
@@ -247,10 +292,9 @@ impl<T: Float + FloatConst + Signed> Quaternion<T> {
         self.imag().norm2()
     }
 
-
     /// generate a Quaternion that represents a rotation of a angle `theta`
     /// around the axis(normalized) `v`
-    pub fn rotation(theta: T, v: V3<T>) -> Self {
+    pub fn rotation(theta: T, v: &V3<T>) -> Self {
         let one = T::one();
         let two = one + one;
         let n   = v.normalize().expect("the input has to be a non zero vector");
@@ -263,32 +307,18 @@ impl<T: Float + FloatConst + Signed> Quaternion<T> {
     /// generate a Quaternion that represents a rotation of a angle `theta`
     /// around the axis(normalized) `v`, the angle `theta` is encoded in the
     /// norm of the vector `v`
-    pub fn rotation_norm_encoded(v: V3<T>) -> Self {
+    pub fn rotation_norm_encoded(v: &V3<T>) -> Self {
         let one = T::one();
         let two = T::from(2.0).unwrap();
         let theta = v.norm2();
-        if !nearly_equal(theta, T::zero(), T::epsilon()) {
+        if !nearly_zero(theta) {
             let (s, c) = (theta / two).sin_cos();
-            Self{q0: c, q: v * (s / theta), normalized: true}
+            Self{q0: c, q: *v * (s / theta), normalized: true}
         } else {
             Self::new(one, V3::zeros())
         }
     }
 
-    pub fn to_rotation(&self) -> M33<T> {
-        let (q0, q) = (self.real(), self.imag());
-        let q0_s = q0 * q0;
-        let (q1, q2, q3) = (q[0], q[1], q[2]);
-        let q1_s = q1 * q1;
-        let q2_s = q2 * q2;
-        let q3_s = q3 * q3;
-        let two = T::one() + T::one();
-
-        m33_new!(q0_s + q1_s - q2_s - q3_s, two*q1*q2 - two*q0*q3, two*q1*q3 + two*q0*q2;
-                 two*q1*q2 + two*q0*q3, q0_s - q1_s + q2_s - q3_s, two*q2*q3 - two*q0*q1;
-                 two*q1*q3 - two*q0*q2, two*q2*q3 + two*q0*q1, q0_s - q1_s - q2_s + q3_s)
-    }
-
     /// create a quaternion that represents the rotation from a Euler angles
     /// with the roll-pitch-yay convention
     pub fn from_euler_angles(yay: T, pitch: T, roll: T) -> Self {
@@ -307,13 +337,13 @@ impl<T: Float + FloatConst + Signed> Quaternion<T> {
 
     /// get the angle of representation from this Quaternion
     pub fn get_angle(&self) -> T {
-        let two = T::from(2.0).unwrap();
+        let one = T::one();
+        let two = one + one;
         let n = self.q.norm2();
 
         two * T::atan2(n, self.q0)
     }
 
-    // TODO(elsuizo:2021-04-15): this is with `sin` or `cos` ???
     /// get the axis of rotation from which this Quaternion represent
     pub fn get_axis(&self) -> Option<V3<T>> {
         let qn = self.normalize()?;
@@ -322,15 +352,12 @@ impl<T: Float + FloatConst + Signed> Quaternion<T> {
     }
 
     /// combine the two previous methods: `get_axis` and `get_angle`
-    pub fn axis_angle(&self) -> (T, Option<V3<T>>) {
-        (self.get_angle(), self.get_axis())
-    }
-
-    /// get the euler angles from the Quaternion
-    pub fn to_euler_angles(&self) -> (T, T, T) {
-        rotation_to_euler(self.to_rotation())
+    pub fn axis_angle(&self) -> (Option<V3<T>>, T) {
+        (self.get_axis(), self.get_angle())
     }
 
+    // TODO(elsuizo:2021-05-20): this epsilon comparison could be wrong maybe we need a
+    // nearly_equal here
     /// normalize the Quaternion
     pub fn normalize_q(&self) -> Self {
         let a = self.dot(*self);
@@ -353,6 +380,7 @@ impl<T: Float + FloatConst + Signed> Quaternion<T> {
         return (result, a)
     }
 
+    /// get the argument of the Quaternion
     pub fn argq(&self) -> Self {
         let result = Quaternion::new(T::zero(), self.q);
         result.normalize_q()
@@ -386,22 +414,6 @@ impl<T: Float + FloatConst + Signed> Quaternion<T> {
         }
     }
 
-    /// sin function apply to the current Quaternion
-    pub fn sin(&self) -> Self {
-        let one = T::one();
-        let two = one + one;
-        let l = self.argq();
-        ((*self * l).exp() - (*self * -l).exp())/ (l * two)
-    }
-
-    /// cos function apply to the current Quaternion
-    pub fn cos(&self) -> Self {
-        let one = T::one();
-        let two = one + one;
-        let l = self.argq();
-        ((*self * l).exp() + (*self * -l).exp()) / two
-    }
-
     /// sqrt function apply to the current Quaternion
     pub fn sqrt(&self) -> Self {
         let one = T::one();
@@ -423,9 +435,11 @@ impl<T: Float + FloatConst + Signed> Quaternion<T> {
     /// https://www.mrpt.org/tutorials/programming/maths-and-geometry/slerp-interpolation/
     ///
     /// Function arguments:
-    /// a: Quaternion(normalized)
-    /// b: Quaternion(normalized)
-    /// t: Float in the closed interval [0.0, 1.0]
+    /// `a`: Quaternion(normalized)
+    ///
+    /// `b`: Quaternion(normalized)
+    ///
+    /// `t`: Float in the closed interval [0.0, 1.0]
     ///
     pub fn slerp(a: Self, b: Self, t: T) -> Self {
         let one = T::one();
@@ -474,6 +488,59 @@ impl<T: Float + FloatConst + Signed> Quaternion<T> {
             return result
         }
     }
+
+    /// Brief.
+    ///
+    /// calculate the instantaneous Quaternion derivative representing a Quaternion rotating at
+    /// rate given by a vector rate
+    ///
+    /// Function arguments:
+    /// `rate`: V3<Float>
+    ///
+    pub fn derivative(&self, rate: &V3<T>) -> Self {
+        let one = T::one();
+        let two = one + one;
+        Self::new_imag(rate) * (one / two) * (*self)
+    }
+}
+
+impl<T: Float + Signed> Quaternion<T> {
+    /// Calculate the inverse of the Quaternion
+    pub fn inverse(&self) -> Option<Self> {
+        if !self.normalized {
+            let norm_sqr = self.abs2();
+            if !nearly_zero(norm_sqr) {
+                Some(self.conj() / norm_sqr)
+            } else {
+                None
+            }
+        } else {
+            Some(self.conj())
+        }
+    }
+
+    /// sin function apply to the current Quaternion
+    pub fn sin(&self) -> Self {
+        let one = T::one();
+        let two = one + one;
+        let l = self.argq();
+        ((*self * l).exp() - (*self * -l).exp())/ (l * two)
+    }
+
+    /// cos function apply to the current Quaternion
+    pub fn cos(&self) -> Self {
+        let one = T::one();
+        let two = one + one;
+        let l = self.argq();
+        ((*self * l).exp() + (*self * -l).exp()) / two
+    }
+}
+
+impl<T: Float + FloatConst> Quaternion<T> {
+    /// get the euler angles from the Quaternion
+    pub fn to_euler_angles(&self) -> (T, T, T) {
+        rotation_to_euler(&self.to_rotation())
+    }
 }
 
 // convert from array to Quaternion
@@ -564,11 +631,11 @@ mod test_quaternion {
     // NOTE(elsuizo:2021-04-14): we assume all the values of the angles in radians!!!
     #[test]
     fn rotate_vec() {
-        let q1 = Quaternion::rotation(90.0f32.to_radians(), V3::new_from(0.0, 0.0, 1.0));
+        let q1 = Quaternion::rotation(90.0f32.to_radians(), &V3::new_from(0.0, 0.0, 1.0));
         let x = V3::new_from(1.0, 0.0, 0.0);
         // rotate x around z 90 degrees
         let result = q1 * x;
-        let expected = V3::new_from(0.0, -1.0, 0.0);
+        let expected = V3::new_from(0.0, 1.0, 0.0);
         assert!(nearly_equal(result[0], expected[0], EPS));
         assert!(nearly_equal(result[1], expected[1], EPS));
         assert!(nearly_equal(result[2], expected[2], EPS));
@@ -576,7 +643,7 @@ mod test_quaternion {
 
     #[test]
     fn rotate_vec_composition_360() {
-        let q1 = Quaternion::rotation(90.0f32.to_radians(), V3::new_from(0.0, 0.0, 1.0));
+        let q1 = Quaternion::rotation(90.0f32.to_radians(), &V3::new_from(0.0, 0.0, 1.0));
         let x = V3::new_from(1.0, 0.0, 0.0);
         // rotate x around z (90 * 4 = 360) degrees
         let result = q1 * q1 * q1 * q1 * x;
@@ -587,10 +654,10 @@ mod test_quaternion {
 
     #[test]
     fn rotate_vec_angle_encode() {
-        let q = Quaternion::rotation_norm_encoded(V3::new_from(0.0, 0.0, 90.0f32.to_radians()));
+        let q = Quaternion::rotation_norm_encoded(&V3::new_from(0.0, 0.0, 90.0f32.to_radians()));
         let x = V3::x_axis();
         let result = q * x;
-        let expected = V3::new_from(0.0, -1.0, 0.0);
+        let expected = V3::new_from(0.0, 1.0, 0.0);
         assert!(nearly_equal(result[0], expected[0], EPS));
         assert!(nearly_equal(result[1], expected[1], EPS));
         assert!(nearly_equal(result[2], expected[2], EPS));
@@ -598,7 +665,7 @@ mod test_quaternion {
 
     #[test]
     fn convert_rotation_test() {
-        let q = Quaternion::rotation_norm_encoded(V3::new_from(0.0, 0.0, 90.0f32.to_radians()));
+        let q = Quaternion::rotation_norm_encoded(&V3::new_from(0.0, 0.0, 90.0f32.to_radians()));
         let x = V3::x_axis();
         // rotate the x around z axis 360 degrees
         let expected = q * q * q * q * x;
@@ -615,7 +682,7 @@ mod test_quaternion {
     #[test]
     fn inverse_test() {
         let q = Quaternion::new_from(1.0, 1.0, 1.0, 10.0);
-        if let Some(inv) = q.inv() {
+        if let Some(inv) = q.inverse() {
             let result = q * inv;
             let expected = Quaternion::one();
             assert!(nearly_equal(result.q0, expected.q0, EPS));
@@ -637,8 +704,8 @@ mod test_quaternion {
 
     #[test]
     fn slerp_test() {
-        let a = Quaternion::rotation(1.78, V3::new_from(1.0, 2.0, 3.0));
-        let b = Quaternion::rotation(1.78, V3::x_axis());
+        let a = Quaternion::rotation(1.78, &V3::new_from(1.0, 2.0, 3.0));
+        let b = Quaternion::rotation(1.78, &V3::x_axis());
         let result = Quaternion::slerp(a, b, 0.3);
         // NOTE(elsuizo:2021-04-24): this result is from julia language
         let expected = Quaternion::new_from(0.6995922116669001, 0.42947374679735195, 0.31677365769795535, 0.475160486546933);
