@@ -31,6 +31,8 @@
 use num::{Float};
 use crate::matrix3x3::M33;
 use crate::traits::LinearAlgebra;
+use crate::matrix4x4::M44;
+use crate::dual_quaternion::DualQuaternion;
 // NOTE(elsuizo:2020-06-02): the following function
 // is a translation of the implementation that is here:
 // https://floating-point-gui.de/errors/comparison/
@@ -54,6 +56,10 @@ pub fn nearly_equal<T: Float>(a: T, b: T, epsilon: T) -> bool {
     }
 }
 
+pub fn nearly_zero<T: Float>(a: T) -> bool {
+    nearly_equal(a, T::zero(), T::epsilon())
+}
+
 // TODO(elsuizo:2020-06-16): this is the only place when i use a Vec
 /// utility function to compare vectors of Floats
 pub fn compare_vecs<T: Float>(v1: &[T], v2: &[T], epsilon: T) -> bool {
@@ -79,6 +85,28 @@ pub fn is_rotation<T: Float + std::iter::Sum>(r: M33<T>) -> bool {
     }
 }
 
+pub fn is_rotation_h<T: Float + std::iter::Sum>(r: M44<T>) -> bool {
+    let r2 = r * r;
+    let eps = T::from(1e-6).unwrap();
+    if nearly_equal(r.det(), T::one(), eps) && nearly_equal(r2.det(), T::one(), eps) {
+        true
+    } else {
+        false
+    }
+}
+
+pub fn compare_dual_quaternions<T: Float>(a: DualQuaternion<T>, b: DualQuaternion<T>, epsilon: T) -> bool {
+    nearly_equal(a.real().real(), b.real().real(), epsilon) &&
+    nearly_equal(a.real().imag()[0], b.real().imag()[0], epsilon) &&
+    nearly_equal(a.real().imag()[1], b.real().imag()[1], epsilon) &&
+    nearly_equal(a.real().imag()[2], b.real().imag()[2], epsilon) &&
+
+    nearly_equal(a.dual().real(),    b.dual().real(), epsilon) &&
+    nearly_equal(a.dual().imag()[0], b.dual().imag()[0], epsilon) &&
+    nearly_equal(a.dual().imag()[1], b.dual().imag()[1], epsilon) &&
+    nearly_equal(a.dual().imag()[2], b.dual().imag()[2], epsilon)
+}
+
 //-------------------------------------------------------------------------
 //                        tests
 //-------------------------------------------------------------------------
