@@ -67,6 +67,10 @@ impl<T: Num + Copy> V6<T> {
         let one = T::one();
         Self::new([one, one, one, one, one, one])
     }
+
+    // copy_element_from(&mut self, v: V3<T>, sector) {
+    //
+    // }
 }
 
 impl<T: Num + Copy + std::cmp::PartialOrd> V6<T> {
@@ -348,7 +352,8 @@ impl<T> From<[T; 6]> for V6<T> {
 //-------------------------------------------------------------------------
 impl<T: Num + fmt::Display> fmt::Display for V6<T> {
     fn fmt(&self, dest: &mut fmt::Formatter) -> fmt::Result {
-        write!(dest, "[{0:^3.2} {1:^3.2} {2:^3.2} {3:^3.2} {4:^3.2} {5:^3.2}]\n", self[0], self[1], self[2], self[3], self[4], self[5])
+        writeln!(dest, "[{0:^3.2} {1:^3.2} {2:^3.2} {3:^3.2} {4:^3.2} {5:^3.2}]",
+                 self[0], self[1], self[2], self[3], self[4], self[5])
     }
 }
 
